% PTE_LS Least-Squares Per Tone Equalizer Design.
%
% [FEQM, D] = pte_ls(X, y, h, L, N, bf)
%
% FEQM: equalizer settings ((N x L ); 
% D: delay
% py: Processed received signal
% pX: referenced Frequence domain input signal
%
% X: Training sequence (N X T) in Frequency domain; 
% y: equalizer input ((N + K - 1) x T); s(2 - K), ..., s(2*N)
% h: channel
% L: number of taps
% N: FFT size
% bf: bar flag

% Copyright (c) 1999-2002 The University of Texas
% All Rights Reserved.
%  
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%  
% The GNU Public License is available in the file LICENSE, or you
% can write to the Free Software Foundation, Inc., 59 Temple Place -
% Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
% World Wide Web at http://www.fsf.org.
%  
% Programmers:	Ming Ding
% Version:        %W% %G% pte_ls.m 1.0 01/15/02
% 
% The authors are with the Department of Electrical and Computer
% Engineering, The University of Texas at Austin, Austin, TX.
% They can be reached at arslan@ece.utexas.edu.
% Ming Ding is also with the Embedded Signal Processing
% Laboratory in the Dept. of ECE., http://anchovy.ece.utexas.edu.

   function [FEQM, D, py, pX] = pte_ls(X, y, h, L, N, bf)

% open a figure for progress bar
   [figHndl statusHndl] = setprogbar('Calculating LS Per Tone ...');     
   
   %CP length
   P = 32;
   %P = 0;  %try training signal without cp.
   
   channel_delay = findchanneldelay(h, P+1);
   D = channel_delay;
   
   % Frame alignment
   y = y(channel_delay+1:length(y));

   % update frame count
   T = floor(length(y)/(N+P));
   
   % S/P
   y = reshape(y(1:(N+P)*T), N + P, T);
 
   % get rid of bad blocks
   X = X(:, 2:(T - 1));
   pX = X;
   y = y(:, 2:(T - 1));
   py = y;
   T = T - 2;

   FEQM=zeros(L,N/2);
   SDFT=zeros(N,L,T);
  
   for j=1:T
      updateprogbar(statusHndl,j,T);
      SDFT(:,:,j)=SlidingDFT(y(:,j),N,L);
      j
   end
   
   for i=1:N/2
   temp=conj(reshape(SDFT(i,:,:),L,T))';
   FEQM(:,i)=pinv(temp)*X(i,:).';    
   end
   
   % close progress bar
   close(figHndl);